"""
Copyright (C) 2021 Clariteia SL

This file is part of minos framework.

Minos framework can not be copied and/or distributed without the express permission of Clariteia SL.
"""


class ImportedClassTest(object):
    def return_test_example(self):
        return "test passed"
